﻿# ******************************************************
# *
# * Name:         managing-pbi-datasets.ps1
# *     
# * Design Phase:
# *     Author:   John Miner
# *     Date:     08-06-2020
# *     Purpose:  Manage Power BI workspaces
# * 
# ******************************************************

# https://docs.microsoft.com/en-us/powershell/power-bi/overview?view=powerbi-ps


#
# Allow scripts to run
#

# Find the currrent level
Get-ExecutionPolicy

# Set for script execution
Set-ExecutionPolicy -ExecutionPolicy Unrestricted


#
# Check on module
#

# Verify module
Get-Command -Module MicrosoftPowerBIMgmt


# Count cmdlets
$CmdCnt = Get-Command | Where-Object { $_.Source -match 'MicrosoftPowerBIMgmt' -and $_.CommandType -match 'Cmdlet' } | Measure-Object -Line
$CmdCnt.Lines

# List cmdlets
Get-Command | Where-Object { $_.Source -match 'MicrosoftPowerBIMgmt.Data'} 


#
# Connect to service
#

clear-host
 
# User credential
$User = "pbi@craftydba.com"
$Pword = ConvertTo-SecureString –String 'W2OL0qeL3A9BrVi0' –AsPlainText -Force
$Credential = New-Object –TypeName System.Management.Automation.PSCredential –ArgumentList $User, $Pword

# Connect to service
Connect-PowerBIServiceAccount -Credential $Credential 



#
#  Create a new workspace
#

$workspace1 = New-PowerBIWorkspace -Name "Push Data Sets"
$workspace1



#
#  Create a new dataset
#

# Create table (prime numbers)
$col1 = New-PowerBIColumn -Name "my_Id" -DataType Int64
$col2 = New-PowerBIColumn -Name "my_Value" -DataType Int64
$col3 = New-PowerBIColumn -Name "my_Division" -DataType Int64
$col4 = New-PowerBIColumn -Name "my_Time" -DataType DateTime
$table1 = New-PowerBITable -Name "tbl_Primes" -Columns $col1, $col2, $col3, $col4


# Create table (fibonacci)
$col5 = New-PowerBIColumn -Name "my_Id" -DataType Int64
$col6 = New-PowerBIColumn -Name "my_Value" -DataType Int64
$col7 = New-PowerBIColumn -Name "my_Time" -DataType DateTime
$table2 = New-PowerBITable -Name "tbl_Fibonacci" -Columns $col5, $col6, $col7


# Create dataset (math)
$dataset1 = New-PowerBIDataSet -Name "ds_Number_Theory" -Tables $table1, $table2

# Add to workspace
$dataset2 = Add-PowerBIDataSet -DataSet $dataset1 -WorkspaceId $workspace1.Id
$dataset2



#
#  Calculate fibonacci numbers
#


# first f(n-2)
[datetime]$date1 = Get-Date -Format "MM/dd/yyyy HH:mm"
[Int64]$f1 = 0
Add-PowerBIRow -DataSetId $dataset2.Id -TableName "tbl_Fibonacci" -Rows @{"my_Id"=1;"my_Value"=$f1;"my_Time"=$date1}

# second f(n-1)
$date1 = Get-Date -Format "MM/dd/yyyy HH:mm"
[Int64]$f2 = 1
Add-PowerBIRow -DataSetId $dataset2.Id -TableName "tbl_Fibonacci" -Rows @{"my_Id"=2;"my_Value"=$f2;"my_Time"=$date1}

# arithmatic overflow at fn(93)
for ($i=0; $i -lt 91; $i++) {

    # new number
    $fn = $f1 + $f2

    # add to row
    $r = $i+3
    Add-PowerBIRow -DataSetId $dataset2.Id -TableName "tbl_Fibonacci" -Rows @{"my_Id"=$r;"my_Value"=$fn;"my_Time"=$date1}

    # continue sequence
    $f1 = $f2
    $f2 = $fn
}



#
#  Calculate prime numbers
#

# first prime number
[datetime]$date1 = Get-Date -Format "MM/dd/yyyy HH:mm"
[Int64]$p = 2
Add-PowerBIRow -DataSetId $dataset2.Id -TableName "tbl_Primes" -Rows @{"my_Id"=1;"my_Value"=$p;"my_Time"=$date1;"my_division"=1}

# second prime number
$date1 = Get-Date -Format "MM/dd/yyyy HH:mm"
[Int64]$p = 3
Add-PowerBIRow -DataSetId $dataset2.Id -TableName "tbl_Primes" -Rows @{"my_Id"=2;"my_Value"=$p;"my_Time"=$date1;"my_division"=1}

# calculate prime numbers
[Int64]$i = 3
[Int64]$n = 4

do
{
  # prime flag
  $f = $true

  # test from 2 to sqrt(n)
  $l = [math]::Sqrt($n) + 1
  for ($j=2; $j -lt [int]$l; $j++) {
      if (($n % $j) -eq 0) { 
        $f = $false
        break; 
      }
  }

  # found a prime number
  if ($f -eq $true)  {
      write-host "prime number "
      write-host $n
      Add-PowerBIRow -DataSetId $dataset2.Id -TableName "tbl_Primes" -Rows @{"my_Id"=$i;"my_Value"=$n;"my_Time"=$date1;"my_division"=$j}
      $i = $i + 1
  }

  # next number to check
  $n = $n + 1
  $f = $true

} while ($i -le 100)




#
# Update table (no loss of data)
#


# all tables
$curTables = Get-PowerBITable -DatasetId $dataset2.Id
$curTables
write-host ""

# fib table
$curTable = $curTables[1]
$curTable
write-host ""

# change schema
$col8 = New-PowerBIColumn -Name "my_Id" -DataType Int64
$col9 = New-PowerBIColumn -Name "my_Value" -DataType Int64
$updTable = New-PowerBITable -Name $curTable.Name -Columns $col8, $col9
$updTable

# update the table
Set-PowerBITable -Table $updTable -DatasetId $dataset2.Id


#
# Various get cmdlets
#

# dataset (imported data)
clear-host
$q1 = Get-PowerBIDataset -Scope Organization -Name "ds_Number_Theory" -WorkspaceId $workspace1.Id
$q1

# table
$q2 = Get-PowerBITable -WorkspaceId $workspace1.Id -DatasetId $q1.Id
$q2



#
#  abf -> premium files
#

# https://docs.microsoft.com/en-us/power-bi/admin/service-premium-large-models

# Update storage type
Set-PowerBIDataset -Id $q1.Id -TargetStorageMode PremiumFiles
$q1


#
#  Show the sql server connection
#

# Get local dataset
$q3 = Get-PowerBIDataset -Scope Individual
$q3

# Get data source info
Get-PowerBIDatasource -DatasetId $q3.Id


#
# Drop all rows
#

# Remove-PowerBIRow -DataSetId 38e2d8d9-e09d-4900-93d0-1ea470f8bf4c -TableName "tbl_Primes"



#
# Drop the dataset
#

<#

# Delete dataset
$Id = $dataset2.Id
$Url = "datasets/$Id"
Invoke-PowerBIRestMethod -Url $Url -Method Delete 

#>




#
#  Data Science (Clustering)
#

# Create table (iris)
$col10 = New-PowerBIColumn -Name "sepal_length" -DataType Double
$col11 = New-PowerBIColumn -Name "sepal_width" -DataType Double
$col12 = New-PowerBIColumn -Name "petal_length" -DataType Double
$col13 = New-PowerBIColumn -Name "petal_width" -DataType Double
$col14 = New-PowerBIColumn -Name "species_id" -DataType Int64
$table3 = New-PowerBITable -Name "tbl_Fishers_Iris" -Columns $col10, $col11, $col12, $col13, $col14


# Create table (iris)
$col15 = New-PowerBIColumn -Name "species_id" -DataType Int64
$col16 = New-PowerBIColumn -Name "species_name" -DataType String
$table4 = New-PowerBITable -Name "tbl_Species_Lkup" -Columns $col15, $col16

# Create dataset (machine learning)
$dataset3 = New-PowerBIDataSet -Name "ds_Machine_Learning" -Tables $table3, $table4

# Add to workspace
$dataset4 = Add-PowerBIDataSet -DataSet $dataset3 -WorkspaceId $workspace1.Id
$dataset4



$workspace1 = Get-PowerBIWorkspace  -Name "Push Data Sets"
$dataset4 = Get-PowerBIDataSet -Name "ds_Machine_Learning" -WorkspaceId $workspace1.Id

# Load data from file
$data = Import-Csv -Path "C:\Users\jminer\Desktop\iris.csv"

<#
for ($i = 0; $i -lt $data.Count; $i++)
{
   
   $item = @{
        "petal_length" =  $data[$i].petal_length;
        "petal_width"  = $data[$i].petal_width;
        "sepal_length" = $data[$i].sepal_length;
        "sepal_width" = $data[$i].sepal_width;
        "species_id" = $data[$i].species
    }

   
}
#>


Add-PowerBIRow -DataSetId $dataset4.Id -TableName "tbl_Fishers_Iris" -Rows $data

# Load from code
Add-PowerBIRow -DataSetId $dataset4.Id -TableName "tbl_Species_Lkup" -Rows @{"species_id"=0;"species_name"="setosa"}
Add-PowerBIRow -DataSetId $dataset4.Id -TableName "tbl_Species_Lkup" -Rows @{"species_id"=1;"species_name"="versicolor"}
Add-PowerBIRow -DataSetId $dataset4.Id -TableName "tbl_Species_Lkup" -Rows @{"species_id"=2;"species_name"="virginica"}



<#

# Delete dataset
$Id = $dataset4.Id
$Url = "datasets/$Id"
Invoke-PowerBIRestMethod -Url $Url -Method Delete 

#>

#
#  Log off
#

# Stop the session
Disconnect-PowerBIServiceAccount




# Reference info
# https://docs.microsoft.com/en-us/powershell/module/microsoftpowerbimgmt.data/get-powerbidatasource?view=powerbi-ps

